/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "html/document.hh"
#include "html/color.hh"
#include "html/genAttr.hh"
#include "html/tag_only.hh"
#include "html/text.hh"
#include "html/writer.hh"

using namespace html;

//====================================== Construct an empty document
document::document(void) 
{
   init_header();
}

document::document(const char* title) 
{
   setTitle(title);
   init_header();
}

document::~document(void) {
}

document*
document::clone(void) const {
   return new document(*this);
}

void
document::init_header(void) {
   mHead << meta("Content-Type", "text/html; charset=iso-8859-1");
}

//======================================  Install scripts ans syle files.
void 
document::install(const char* from, const char* to) const {
   header().install(from, to);
   compound::install(from, to);
}

void 
document::write(writer& out) const {
   out.tag("HTML");

   //----------------------------------  Add header section and meta-data
   mHead.write(out);
   out.endLine();

   //----------------------------------  Add style dictionary
   if (!mStyle.empty()) {
      mStyle.write(out);
      out.endLine();
   }

   //----------------------------------  Add body
   compound::write(out);
   out.endTag("HTML");
}

//======================================  Get the tag
const char *
document::getTag(void) const {
   return "BODY";
}

//======================================  Set the background color.
void 
document::setBackgroundColor(const color& clr) {
   compound::repAttr("BGCOLOR", clr);
}

//======================================  Set the refresh period
void 
document::setRefresh(double t) {
   if (t != 0) {
      mHead << meta("Refresh", std::to_string(t));
   }
}

//======================================  Set the dcument title
void 
document::setTitle(const std::string& doc_ttl) {
   mHead << title(doc_ttl);
}
